###################################################################################################
###################################################################################################
############### SHORT COURSE ON COALESCENT SIMULATIONS AND ABC INFERENCE ESTIMATION ###############
###############                   BASIC FUNCTIONS AND EXERCISES ON R                ###############
###############                        Sebastian E. Ramos-Onsins                    ###############
###############      Centre for Research in Agricultural Genomics (CRAG) Spain      ###############
###################################################################################################
###################################################################################################


###################################################################################################
###############  2.1	Calculate the time to the ancestor with n = 2 and Ne = 1000 ###############
###################################################################################################
#Imagine a sample of two individuals and 1000 total (haploid) individuals. The probability that these two individuals have the same parent one generation ago is 1/(1000). We can calculate the number of generations until having the same parent. This is a stochastic process, which means that the result is different each run. The function is:
###################################################################################################

time_coal2_generations <- function(Ne) {
	tg <- 0
	coalescence <- 0
	pcoal <- 1/(Ne)
	while(coalescence == 0) {
		tg <- tg + 1
		if(runif(1) <= pcoal) coalescence <- 1	
	}
	tg	
}

############### EXERCISES SECTION 2.1:#######################
#QUESTION: Calculate the time to coalescence. Do 1000 replicates and see the differences in the values.
#par(mfrow=c(1,2))
#ANSWER 1: Coalecent time for 2 samples, Ne=1000. Do 1000 replicates:
	t2g <- array(dim=c(1000))
	for(i in 1:1000) {
		t2g[i] <- time_coal2_generations(Ne=1000) #function fom section 2.1
	}
	mean(t2g)
	var(t2g)
	hist(t2g,breaks=30)

#QUESTION: What happens if you increase/reduce the Ne?
#ANSWER: The times become smaller for Ne small and larger for Ne large

How you can modify the function to include a sudden change in population size 100 generations ago?
#Note that the new function includes a conditional that changes the Ne and therefore the probability of coalescence.
#function
time_coal2_generations_sudden <- function(Ne,t,f) {
	tg <- 0
	coalescence <- 0
	pcoal <- 1/(Ne)
	while(coalescence == 0) {
		tg <- tg + 1
		if(t==tg) pcoal <- 1/(Ne*f)
		if(runif(1) <= pcoal) coalescence <- 1
	}
	tg	
}

#run the process for a sudden change in population size 100 generations ago of 100 total individuals
t2g <- array(dim=c(1000))
for(i in 1:1000) {
	t2g[i] <- time_coal2_generations_sudden(Ne=1000,t=100,f=0.1) #function fom section 2.1
}
mean(t2g)
var(t2g)
hist(t2g,breaks=30)

###################################################################################################
################ 2.2	Calculate the time to the ancestor with n = 20 and Ne = 1000 ##############
###################################################################################################
#For a sample of 20 individuals, the first coalescence is, as an average, much more early, because the probability of having an ancestor is bigger: ∼ (20 ∗ 19/2)/1000. The function is:
###################################################################################################

time_coaln_generations <- function(Ne,n) {
	tg <- 0
	coalescence <- 0
	pncoal <- 1
	for(i in 1:(n-1)) pncoal <- pncoal * (1 - i/(Ne))
	pcoal <- 1 - pncoal
	while(coalescence == 0) {
		tg <- tg + 1
		if(runif(1) <= pcoal) coalescence <- 1	
	}
	tg	
}

#This probability across the generations follows a exponential distribution. Then, we can simplify the function and increase the speed for the calculation. The command chose(n,2) is used to calculate n over 2:

time_coaln_fast_generations <- function(Ne,n) {
	t_rate <- choose(n,2)/(Ne)
	#take a random value fom 0 to 1
	t1 <- qexp(p=runif(1),rate=t_rate)
	t1	
}

##################EXERCISES SECTION 2.2:#########################
#QUESTION: Calculate the mean and the variance in 1000 replicates for each function (generation by generation and the fast one) and compare the values. Use n=2 and n=20, Ne=1000

#ANSWER: First Coalecent time for 4 samples, Ne=1000. Do 1000 replicates:
#function 2.2.1. First Coalecent time for n=2 samples, Ne=1000. Do 1000 replicates:
	par(mfrow=c(1,2))

	t20g <- array(dim=c(1000))
	time1 <- Sys.time()
	for(i in 1:1000) {
		t20g[i] <- time_coaln_generations(Ne=1000,n=2) #function fom section 2.2
	}
	mean(t20g)
	var(t20g)
	hist(t20g,breaks=30)
	time2 <- Sys.time()
	ttimesg <- as.numeric(difftime(time2,time1,units="secs"))
	ttimesg #Time in seconds in calculate the result

#function 2.2.2. Coalescent time for n=2 samples. , Ne=1000. Do 1000 replicates:
	t20g <- array(dim=c(1000))
	time1 <- Sys.time()
	for(i in 1:1000) {
		t20g[i] <- time_coaln_fast_generations(Ne=1000,n=2) #function fom section 2.2
	}
	mean(t20g)
	var(t20g)
	hist(t20g,breaks=30)
	time2 <- Sys.time()
	ttimesg <- as.numeric(difftime(time2,time1,units="secs"))
	ttimesg #Time in second in calculate the result
#Compare the values of time but also the speed it needs to finish. Look that the distribution may not be completely equal with the continuous approach


###################################################################################################
############ 2.3	Calculate the time to the ancestor for a large (and unknown) Ne ###############
###################################################################################################
#The normal situation is that the Effective number of individuals is unknown, so we are unable to include this value in the simulations. We also assume that the population is large. Also, the sample must be much smaller than the population size. Then, the number of generations calculated is a function of Ne:
###################################################################################################

time_coaln_fast_Ne <- function(n) {
	t_rate <- choose(n,2)/1
	t1 <- qexp(p=runif(1),rate=t_rate)
	t1	
}

######################EXERCISES SECTION 1.3:#######################
#QUESTION 2.3: Modify the function to include a sudden change (0.01x) in population size 0.1*Ne generations ago.

#Modify the previous function: We include a conditional:
time_coaln_fast_Neft <- function(n,f=1,t=1e6) {
	t_rate <- choose(n,2)/1
	t1 <- qexp(p=runif(1),rate=t_rate)
	if(t1 >= t) {
		t_rate <- choose(n,2)/f
		t1 <- t + qexp(p=runif(1),rate=t_rate)
	}
	t1	
}
#Example:
time_coaln_fast_Neft(n=2,f=0.01,t=0.1)

	t2 <- array(dim=c(1000))
	time1 <- Sys.time()
	for(i in 1:1000) {
		t2[i] <- time_coaln_fast_Neft(n=2,f=0.01,t=0.1) #function fom section 2.2
	}
	mean(t2)
	var(t2)
	hist(t2,breaks=30)

###################################################################################################
##### 2.4	Simulate a sample from the simplest model: the Standard Neutral Model (SNM) ###########
###################################################################################################
##### 2.4.1	Do a Coalescent Tree                                                        ###########
###################################################################################################
# We can use the same function than in the previous section (obtain the times of coalescence), and calculate the relationships among lineages (constructing the tree):
###################################################################################################

######################EXERCISES SECTION 2.4:#############################
#ALGORITHM: 
#Do a Coalescent tree for a simple evolutionary model: SNM. Calculate S, pi and theta
	n <- 20
	theta <- 10
	
	#calculate times
	time <- rep(0,n)
	for(i in n:2) {
		time[i-1] <- time_coaln_fast_Ne(i)
	} 
	time
	#calculate relationships: my particular approach
	#initialize the number of the node at each coalescence event time
	init <- rep(1,(n+1))
	for(i in 2:(n+1)) {init[i] <- init[i-1]+(i-1)}
	#the number of nodes after coalescence (from 1 to n)
	init
	#calculate the frequencies: the vector starts to count from the end  
	freqs <- array(0,dim=c(sum(1:n)))
	#at present each node contains one sample
	freqs[init[n]:(init[n+1]-1)] <- 1
	#at each coalescent event, two samples join randomly (panmixia is assumed)
	for(i in (n-1):2) {
		#at event, say j, for a sample of i, pick i-2 samples from the previous coalescent event and keep the value
		freqs[init[i]:(init[i+1]-2)] <- sample(freqs[init[i+1]:(init[i+1]+i)],i-1,replace=F)
		#the last value of the sample will be the the value which sum is n 
		freqs[init[i+1]-1] <- n - sum(freqs[init[i]:(init[i+1]-2)])
	}
	freqs[1] <- n
	#the list of samples per node for each coalescent event is the following
	for(i in n:1) show(freqs[init[i]:(init[i+1]-1)])	

#QUESTION 2.4.1:Calculate the time from present to the common ancestor (note that the times are from the previous event and not from present). Calculate the total sum of branches in the tree (Ttot)

T.pres <- rep(0,n)
T.pres[n-1] <- time[n-1]
for(i in (n-1):2) T.pres[i-1] <- T.pres[i] + time[i-1]
#time the coalescent occurred
T.pres[-n]
#Time from present to the common ancestor
sum(T.pres)

#Total sum of branches in the tree
Ttot <- 0
for(i in n:2) Ttot <- Ttot + i*time[i-1]
#total sum of the time of all branches
Ttot

###################################################################################################
############################# 2.4.2	Include Mutations                 ###########################
###################################################################################################
#The mutations are random events that occur across the time until the coalescence. The process can be simulated by using a Poisson distribution given the population mutation rate and the length of the branches. Here the number of mutations per coalescent event are calculated given using a Poisson distribution with λ = tμ = Tθ/2
###################################################################################################

	#calculate the mutations per branch
	muts <- array(0,dim=c(sum(1:n)))
	for(i in n:2) {
		muts[init[i]:(init[i]+i-1)] <- rpois(i,lambda=time[i-1]*theta/2)
	}
	
###################################################################################################
######################### 2.4.3	Calculate the Frequency Spectrum       ###########################
###################################################################################################
	
	#calculate the frequency spectrum
	freq.spectrum <- array(0,dim=c(n))
	for(i in 1:sum(1:n)) {
		freq.spectrum[freqs[i]] <- freq.spectrum[freqs[i]] + muts[i]
	}
	freq.spectrum

#QUESTION 2.4.3: Take the code of this section (Coalescent Tree + Mutations + Frequency Spectrum) to construct a single function that gives as an output the values of the frequency spectrum.

calculate_freqspectrum_coalescence <- function(n,mutpop) {
	#calculate times
		time <- rep(0,n-1)
		Ttot <- 0
		for(i in n:2) {
			time[i-1] <- time_coaln_fast_Ne(i)
			Ttot <- Ttot + time[i-1]
		}
		#calculate relationships between lineages
		#find the number of the first node for each coalescence event
		init <- rep(1,(n+1))
		for(i in 2:(n+1)) {init[i] <- init[i-1]+(i-1)}
		#calculate the frequencies: the vector starts to count from the end (like coalescent goes from present to past) 
		freqs <- array(0,dim=c(sum(1:n)))
		#at present each node contains one sample
		freqs[init[n]:(init[n+1]-1)] <- 1
		#at each coalescent event, two samples join randomly (panmixia is assumed)
		for(i in (n-1):2) {
			#at event, say j, for a sample of i, pick i-2 samples from the previous coalescent event and keep the value
			freqs[init[i]:(init[i+1]-2)] <- sample(freqs[init[i+1]:(init[i+1]+i)],i-1,replace=F)
			#the last value of the sample will be the the value which sum is n 
			freqs[init[i+1]-1] <- n - sum(freqs[init[i]:(init[i+1]-2)])
		}
		freqs[1] <- n
		#calculate the mutations per branch
		muts <- array(0,dim=c(sum(1:n)))
		for(i in n:2) {
			muts[init[i]:(init[i]+i-1)] <- rpois(i,lambda=time[i-1]*mutpop/2)
		}
	#calculate the frequency spectrum
		freq.spectrum <- array(0,dim=c(n))
		for(i in 1:sum(1:n)) {
			freq.spectrum[freqs[i]] <- freq.spectrum[freqs[i]] + muts[i]
		}
	freq.spectrum
}

###################################################################################################
#### 2.4.4	Calculate estimates of variability based on the frequency spectrum ################### 
###################################################################################################
#Different estimates of 2Ne μ are calculated knowing that a unbiased estimator of θb = iξi , where ξi is the number of mutations at frequency i (Fu, 1995; Achaz, 2009). Different combinations of weights can be used, giving more or less importance to different sections of the frequency spectrum distribution.
###################################################################################################

#Function: calculate S,Watterson theta, nucleotide diversity and others.
calculate_variability_estim <- function(n,freq.spectrum) {
	thetaw <- 0
	thetat <- 0
	thetah <- 0
	thetal <- 0
	for(i in 1:(n-1)) {
		thetaw <- thetaw + freq.spectrum[i] * i * 1/i
		thetat <- thetat + freq.spectrum[i] * i * (n-i)
		thetah <- thetah + freq.spectrum[i] * i * i
		thetal <- thetal + freq.spectrum[i] * i * 1
	}
	thetaw <- thetaw/sum(1/(1:(n-1)))
	thetat <- thetat/sum(n-(1:(n-1)))
	thetah <- thetah/sum(1:(n-1))
	thetal <- thetal/n
	theta1 <- freq.spectrum[1]
	S <- sum(freq.spectrum)
	
	l <- c(S=S,thetaW=thetaw,thetaT=thetat,theta1=theta1,thetaH=thetah,thetaL=thetal)
}

#QUESTION 2.4.4: Use a sample of n = 4 and the estimate of θ = 10 to construct a coalescent tree. Then, calculate the frequency spectrum and variability statistics.

	freq.spectrum <- calculate_freqspectrum_coalescence(n=4,mutpop=10)
	freq.spectrum
	#calculate statistics (estimates of the level of variability)
	stats <- calculate_variability_estim(n=4,freq.spectrum=freq.spectrum)
	stats

###################################################################################################
############## 2.5	Simulate 1000 samples in a Contraction Population Scenario #################### 
###################################################################################################

############################EXERCISES SECTION 2.5: #################################
#QUESTION 2.5.1: Do the same than in the previous section but using a new function for calculating the coalescent times (including the possibility to do a sudden change in population size at time t).

time_coaln_fast_Nef <- function(n,f=1) {
	t_rate <- choose(n,2)/f
	t1 <- qexp(p=runif(1),rate=t_rate)
	t1	
}

calculate_freqspectrum_coalescence_f <- function(n,mutpop,tb=0,sb=1) {
	#calculate times
		time <- rep(0,n-1)
		Ttot <- 0
		f <- 1
		for(i in n:2) {
			t <- time_coaln_fast_Nef(i,f)
			if(Ttot+t > tb && Ttot <= tb) {
				t <- tb - Ttot
				f <- sb
				t <- t + time_coaln_fast_Nef(i,f)
			} 
			time[i-1] <- t
			Ttot <- Ttot + time[i-1]
		}
		#calculate relationships between lineages
		#find the number of the first node for each coalescence event
		init <- rep(1,(n+1))
		for(i in 2:(n+1)) {init[i] <- init[i-1]+(i-1)}
		#calculate the frequencies: the vector starts to count from the end (like coalescent goes from present to past) 
		freqs <- array(0,dim=c(sum(1:n)))
		#at present each node contains one sample
		freqs[init[n]:(init[n+1]-1)] <- 1
		#at each coalescent event, two samples join randomly (panmixia is assumed)
		for(i in (n-1):2) {
			#at event, say j, for a sample of i, pick i-2 samples from the previous coalescent event and keep the value
			freqs[init[i]:(init[i+1]-2)] <- sample(freqs[init[i+1]:(init[i+1]+i)],i-1,replace=F)
			#the last value of the sample will be the the value which sum is n 
			freqs[init[i+1]-1] <- n - sum(freqs[init[i]:(init[i+1]-2)])
		}
		freqs[1] <- n
		#calculate the mutations per branch
		muts <- array(0,dim=c(sum(1:n)))
		for(i in n:2) {
			muts[init[i]:(init[i]+i-1)] <- rpois(i,lambda=time[i-1]*mutpop/2)
		}
	#calculate the frequency spectrum
		freq.spectrum <- array(0,dim=c(n))
		for(i in 1:sum(1:n)) {
			freq.spectrum[freqs[i]] <- freq.spectrum[freqs[i]] + muts[i]
		}
	freq.spectrum
}

#QUESTION 2.5.2: Do 1000 replicates for t = 0.1, Nef = 10, θ = 10, and keep the values of the statistics in a matrix.

	#Do 1000 replicates of a model with sudden contraction and calculate the variability
	niter <- 1000
	n <- 20
	time.con <- 0.1
	strength <- 1e+2 #1
	theta <- 10
	
	frqsp <- array(dim=c(niter,n))
	estim.var <- array(,dim=c(niter,6))
	colnames(estim.var)[1:6] <- c("S","theta.Watt","theta.Taj","theta.FL","theta.FW","theta.Z")
	for(i in 1:niter) {
		freq.spectrum <- calculate_freqspectrum_coalescence_f(n=n,mutpop=theta,tb=time.con,sb=strength)
		variability <- calculate_variability_estim(n=n,freq.spectrum=freq.spectrum)
		frqsp[i,1:n] <- freq.spectrum[1:n]
		estim.var[i,1:6] <- variability[1:6]
	}
	#frqsp
	#estim.var
	mean.frqsp <- apply(frqsp,2,mean)
	mean.estim <- apply(estim.var,2,mean)
	names(mean.estim)[1:6] <- c("S","theta.Watt","theta.Taj","theta.FL","theta.FW","theta.Z")
	t(mean.frqsp)
	mean.estim
	par(mfrow=c(1,3))
	plot(c(1:(n-1)),mean.frqsp[1:(n-1)],type="l",main=c("Mean of Simulated frequency spectrum (black line)\nFrequency spectrum under SNM (dotted line)"),ylim=c(0,max(mean.estim[2],mean.frqsp)))
	#plot(c(1:(n-1)),mean.frqsp[1:(n-1)],type="s",main=c("Mean of Simulated frequency spectrum (black line)\nFrequency spectrum under SNM (dotted line)"),ylim=c(0,max(mean.estim[2],mean.frqsp)))
	#lines(c(1:(n-1)),mean.frqsp[1:(n-1)],type="h")
	lines(mean.estim[2]/(1:(n-1)),type="l",lty=3)
	
#QUESTION 2.5.3: Calculate the 2.5%, 50% and 97.5% quantiles and the mean for each statistic and plot the distributions.

	apply(estim.var,2,quantile,prob=c(0.025,0.5,0.975))
	apply(estim.var,2,mean)
	
	dtwa <- density(estim.var[,2])
	dtta <- density(estim.var[,3])
	dtfl <- density(estim.var[,4])
	dtfw <- density(estim.var[,5])
	dtze <- density(estim.var[,6])
	max.x <- max(dtwa$x,dtta$x,dtfl$x,dtfw$x,dtze$x)
	max.y <- max(dtwa$y,dtta$y,dtfl$y,dtfw$y,dtze$y)
	
	plot(dtwa,col="black",main=sprintf("Distribution of theta values in simulations: \nWatterson(black), Pi(red), \nFu&Li(green), Fay&Wu(blue), Zeng(brown)"),xlim=c(0,max.x),ylim=c(0,max.y))
	lines(dtta,col="red")
	lines(dtfl,col="green")
	lines(dtfw,col="blue")
	lines(dtze,col="brown")

#QUESTION 2.5.4: Are the different estimators having the same result? Are they very different to the value included in the simulations (θ = 10)? 

#QUESTION 2.5.5: Plot differences of different theta estimators: for example, thetaW-thetaTaj is the numerator of Tajima's D statistic.

	dTD <- density(estim.var[,3]-estim.var[,2]) #Numerator of Tajima's D 
	dFL <- density(estim.var[,2]-estim.var[,4]) #Numerator of Fu and Li's D
	dFW <- density(estim.var[,3]-estim.var[,5]) #Numerator of Fay and Wu's H
	max.x <- max(dTD$x,dFL$x,dFW$x)
	max.y <- max(dTD$y,dFL$y,dFW$y)
	min.x <- min(dTD$x,dFL$x,dFW$x)
	min.y <- min(dTD$y,dFL$y,dFW$y)
	
	plot(dTD,col="red",main=sprintf("Distribution of comparison theta values in simulations: \nNumerator of Tajima's D(red)\n Num. Fu&Li's D(green) Num. Proxy of Fay&Wu's H(blue)"),xlim=c(min.x,max.x),ylim=c(min.y,max.y))
	lines(dFL,col="green")
	lines(dFW,col="blue")
	abline(v=0,col="black")

#QUESTION 2.5.5: Repeat the analysis with strength = 1 (SNM) and strength = 1e-1 (Expansion)

#That is, copy from 2.5.2 but changing the initial parameters to:
	niter <- 1000
	n <- 20
	time.con <- 0.1
	strength <- 1 #SNM
	theta <- 10
#and with:
	niter <- 1000
	n <- 20
	time.con <- 0.1
	strength <- 1e-1 #Expansion
	theta <- 10

#############################SECTION 2.6:#########################################
#2.6	Approximate Bayesian estimation of the mutation population parameter θ = 2Neμ in a population
#One of the fundamental analysis in the study of the variability (but also in other fields) is to understand how the observed data fits in a theoretical model (Model-base analysis). In case studying complex bi- ological systems, the fit of the observed data in a model may be very challenging, given the difficulty in obtaining the exact probabilities of the observed data. The Approximate Bayesian Analysis try to solve this problem, simply fitting the data without calculating the exact probabilities (likelihoods). This method uses summary statistics to see how close are the simulated data from the observed data. There is abundant bibliography in the last years about this approximated methodology (see for example Beau- mont et al., 2002; Csill ́ery et al., 2010).
#In the following examples we estimate estimate parameters using the ABC method, although the com- plexity of the models are clearly reduced.
#######################################################################################
#2.6.1	Coalescent Simulation using prior distributions in parameters
#A simple Rejection Algorithm is used to estimate the (unique) parameter of the model. The first step of this estimation method is simulate many replicates of the observed data. The simulation follow a chosen theoretical model (here the SNM). Here the SNM has as a unique parameter: θ. This parameter will have many different values, according to a prior uniform distribution. that mean that different values are used in each of the replicates in the simulation:
#######################################################################################

#Functions:
distance.euclidean <- function(nstats,sim.data,obs.data) {
	#calculate a distance (normalized euclidean) between observed and simulated statistics: first normalize
	scaled.sim.data <- sim.data
	for(k in 1:nstats){
		scaled.sim.data[,k] <- normalise(sim.data[,k],sim.data[,k])
	}
	#normalise observed values
	scaled.obs.data <- obs.data
	for(k in 1:nstats){
		scaled.obs.data[k] <- normalise(obs.data[k],sim.data[,k])
	}
	#calculate distance
	sum1 <- array(0,dim=c(length(sim.data[,1])))
	for(k in 1:nstats) {
		sum1 <- sum1 + (scaled.sim.data[,k]-scaled.obs.data[k])^2
	}
	dst <- sqrt(sum1)
	dst
}

normalise <- function(x,y){
	if(var(y,na.rm=T) == 0)return (x - mean(y,na.rm=T))
	(x-(mean(y,na.rm=T)))/sqrt(var(y,na.rm=T))
}

#ALGORITHM:
#We want to estimate the parameter "theta" under SNM:
#Do: Estimate parameters of variability using ABC Rejection Algorithm
	#Observed data
	n <- 20
	nstats <- 1
	
	#Simulated data: SNM. We have 1 parameter: mutpop.
	time1 <- Sys.time() #time of the process
	
	#define min and max values for the priors
	theta.min <- 1
	theta.max <- 100
	
	#run simulation
	niter <- 25000
	sim.data <- array(dim=c(niter,nstats))
	colnames(sim.data)[1:nstats] <- c("theta.Watt")#,"theta.Taj","theta.FL","theta.FW")

	#pick parameter(s) from the initial distribution
	theta <- runif(niter,theta.min,theta.max)
	
	for(i in 1:niter) {	
		#do coalescent tree and calculate statistics
		freq.spectrum <- calculate_freqspectrum_coalescence(n=n,mutpop=theta[i])
		variability <- calculate_variability_estim(n=n,freq.spectrum=freq.spectrum)
		sim.data[i,1:nstats] <- variability[2:(nstats+1)]
	}
	
#######################################################################################
#2.6.2. ABC: Approximate Bayesian Calculation
#Once we have a number of simulated replicates calculated given different values of the parameter θ, let’s chose those replicates were the simulated Watterson theta’s is closer to observed one:
#######################################################################################

	#calculate euclidean distance between observed and simulated data
	obs.data <- c(thetaW=14.1)
	dst <- distance.euclidean(nstats,sim.data,obs.data)

	#ABC: tolerance at 1%
	tol <- 0.01

	#Accept or reject parameters according to the threshold (tolerance)
    abstol <- quantile(dst,tol,na.rm=TRUE)
    wt1 <- dst < abstol
	
	time2 <- Sys.time()
	ttimesg <- as.numeric(difftime(time2,time1,units="secs"))
	ttimesg

#2.6.3 Results
#The results show how to plot two distributions in the same plot (using lines(). Also the quantiles of the posterior distribution (in blue cyan in the Figure 3) are shown.

	#Show Results
	par(mfrow=c(1,1))
	prior.plot <- hist(x=theta     ,breaks=20,plot=F)
	post.plot  <- hist(x=theta[wt1],breaks=20,plot=F)
	maxy <- max(prior.plot$density,post.plot$density)
	xlimP <- c(theta.min,theta.max)
	ylimP <- c(0,maxy+maxy*0.05)
	plot(post.plot,freq=F,main="Prior and Posterior distribution of theta",xlim=xlimP,ylim=ylimP,col="cyan")
	lines(prior.plot,freq=F,lty=3,lwd=1)
	mean(theta[wt1])
	quantile(theta[wt1],prob=c(0,0.025,0.5,0.975,1))
	

################################SECTION 2.7:#########################################
#2.7	Model Selection: Select the best model (Stationary versus Expan- sion model) given observed data
#Model section refers to a the comparison and selection of two or more models for a better fitting with the observed data. In this section a simple model selection algorithm is explained and results are shown. Other methods of model selection are explained in bibliography (see for example the methods for model selection using ABC in Beaumont, 2008; ?; ?).
######################################################################################
#2.7.1	Coalescent Simulations using prior distributions in parameters for the two Models
#First we simulate our data under two different models: the SNM (one parameter, θ, and a sudden expansion model with three parameters (θ, Texp, f). Each simulation model is calculated separately, using each parameter a range of values according a prior distribution:
######################################################################################

#ALGORITHM:
#Do: Model Selection: SNM vs Expansion given an observed data

	#Observed data
	n <- 20
	nstats <- 4
	obs.data <- c(thetaW=14.1,thetatT=9.4,theta1=28,thetaH=5.7)
	
	#We have two models: SNM and Expansion
	#We simulate the two models separately
	time1 <- Sys.time()
	niter <- 25000
	sim.data.model <- array(dim=c(niter*2,nstats+1))
	colnames(sim.data.model)[1:(nstats+1)] <- c("model","theta.Watt","theta.Taj","theta.FL","theta.FW")
	
	####SNM: 1 parameter: mutpop####
	#define min and max values for the prior
	theta.snm.min <- 1
	theta.snm.max <- 100
	
	#run simulation
	sim.data <- array(dim=c(niter,nstats))

	#pick parameter(s) from the initial distribution
	theta.snm <- runif(niter,theta.snm.min,theta.snm.max)
	
	for(i in 1:niter) {	
		#run SNM model and calculate statistics
		freq.spectrum <- calculate_freqspectrum_coalescence(n=n,mutpop=theta.snm[i])
		sim.data[i,1:nstats] <- calculate_variability_estim(n=n,freq.spectrum=freq.spectrum)[2:(nstats+1)]
	}
	sim.data.model[1:niter,] <- cbind(1,sim.data[1:niter,])

	####Expansion: 3 parameters: mutpop,time.exp,strength#####
	#define min and max values for the priors
	theta.exp.min <- 1
	theta.exp.max <- 100
	time.exp.min  <- 0
	time.exp.max  <- 1
	strg.exp.min  <- log10(1e-6)
	strg.exp.max  <- log10(1)
	
	#run simulations
	sim.data <- array(dim=c(niter,nstats))

	#pick parameter(s) from the initial distribution
	theta.exp <- runif(niter,theta.exp.min,theta.exp.max)
	time.exp <- runif(niter,time.exp.min,time.exp.max)
	strg.exp <- runif(niter,strg.exp.min,strg.exp.max)
	strg.exp <- 10^(strg.exp)
	
	for(i in 1:niter) {	
		#run exponential model and calculate statistics
		freq.spectrum <- calculate_freqspectrum_coalescence(n=n,mutpop=theta.exp[i],tb=time.exp[i],sb=strg.exp[i])
		sim.data[i,1:nstats] <- calculate_variability_estim(n=n,freq.spectrum=freq.spectrum)[2:(nstats+1)]
	}
	sim.data.model[(niter+1):(2*niter),] <- cbind(2,sim.data[1:niter,])

######################################################################################
#2.7.2	Contrasting Models using ABC
#Model Selection is obtained simply by a Rejection method, where all iterations are included in the same pool and only the closest (1%) distances to the observed data are selected. Then, we count the times the model 1 (SNM) or 2 (Expansion model) are chosen. A large difference, say more than 5 times, is considered a good support in favour to the more frequently selected.
######################################################################################

	##### DO ABC Model Selection: ########
	#Model Selection:
	#calculate euclidean distance between observed and simulated data
	dst <- distance.euclidean(nstats,sim.data.model[,2:(1+nstats)],obs.data)

	#Accept or reject parameters according to the threshold (tolerance)
	#ABC: tolerance at 1%
	tol <- 0.01
    abstol <- quantile(dst,tol,na.rm=TRUE)
    wt1 <- dst < abstol
	
	snm <- sum(sim.data.model[wt1,1]==1)
	exp <- sum(sim.data.model[wt1,1]==2)
	
	time2 <- Sys.time()
	ttimesg <- as.numeric(difftime(time2,time1,units="secs"))
	ttimesg

#2.7.3 Results
#Show the results for the selection model. Also, once the model is selected, it is also important to estimate the posterior distribution of the parameters of the model:

	#Show Results
	sprintf("Number of SNM values accepted: %.0f",snm)
	sprintf("Number of Exponential values accepted: %.0f",exp)
	sprintf("Ratio Exp/SNM: %.3f",exp/snm)
	
	#wt2 are the choosen values in the Exponential model
	wt2 <- sim.data.model[(niter+1):(2*niter),1]==2 & wt1[(niter+1):(2*niter)]
	

######################################################################################
#Exercise 2.7.3: Show the histograms and the quantiles for the parameter(s) of the selected model. Change the tolerance and compare the results.
######################################################################################

#SOLUTION to QUESTION 2.7.3: 

	#Posterior distributions for the Exponential Model
	par(mfrow=c(1,3))
	prior.plot <- hist(x=theta.exp     ,breaks=20,plot=F)
	post.plot  <- hist(x=theta.exp[wt2],breaks=20,plot=F)
	maxy <- max(prior.plot$density,post.plot$density)
	xlimP <- c(theta.exp.min,theta.exp.max)
	ylimP <- c(0,maxy+maxy*0.05)
	plot(post.plot,freq=F,main="Prior and Posterior distribution of theta for Exponential model",xlim=xlimP,ylim=ylimP,col="cyan")
	lines(prior.plot,freq=F,lty=3,lwd=1)
	mean(theta.exp[wt2],na.rm=T)
	quantile(theta.exp[wt2],prob=c(0,0.025,0.5,0.975,1),na.rm=T)

	prior.plot <- hist(x=time.exp     ,breaks=20,plot=F)
	post.plot  <- hist(x=time.exp[wt2],breaks=20,plot=F)
	maxy <- max(prior.plot$density,post.plot$density)
	xlimP <- c(time.exp.min,time.exp.max)
	ylimP <- c(0,maxy+maxy*0.05)
	plot(post.plot,freq=F,main="Prior and Posterior distribution of time for Exponential model",xlim=xlimP,ylim=ylimP,col="cyan")
	lines(prior.plot,freq=F,lty=3,lwd=1)
	mean(time.exp[wt2],na.rm=T)
	quantile(time.exp[wt2],prob=c(0,0.025,0.5,0.975,1),na.rm=T)

	prior.plot <- hist(x=log10(strg.exp)     ,breaks=20,plot=F)
	post.plot  <- hist(x=log10(strg.exp[wt2]),breaks=20,plot=F)
	maxy <- max(prior.plot$density,post.plot$density)
	xlimP <- c(strg.exp.min,strg.exp.max)
	ylimP <- c(0,maxy+maxy*0.05)
	plot(post.plot,freq=F,main="Prior and Posterior distribution of strength for Exponential model",xlim=xlimP,ylim=ylimP,col="cyan")
	lines(prior.plot,freq=F,lty=3,lwd=1)
	mean(strg.exp[wt2],na.rm=T)
	quantile(strg.exp[wt2],prob=c(0,0.025,0.5,0.975,1),na.rm=T)


###################################################################################################
###################################################################################################
###################################################################################################

#3. Inference of Evolutionary parameters and Model Choice Analysis using ABC with ms and the R library abc

###################################################################################################
###################################################################################################
###################################################################################################
###################################################################################################

#In this exercise we assume that we have a sample of n = 20 and the following estimates of variability:

#	S = 50 
#	Theta(Watterson) = 14.1 
#	Theta(Tajima) = 9.4 
#	Theta(Fay&Wu) = 5.7

#We want to know wether the Standard Neutral Model (SNM) or a model including a Demographic Expansion (DEM) explains better the observed data.
#To do this exercise, we will use the coalescent simulator ms (Hudson, 2002) and the R library abc (Csill ́ery et al., 2010) to perform Approximate Bayesian Computation Analysis (Beaumont et al., 2002; Beaumont, 2008).

library(abc)
niter <- 50000
OBS <- c(9.4,50,5.7) #Pi,S,H

###################################################################################################
#3.1	Simulate statistics under SNM with variable parameter θ 
###################################################################################################

#First we need to create a large number (niter=50000) of values of θ given a distribution (here we will
use a Uniform between 1 and 100). Go to R:

#### SNM: 1 parameter: theta ####
theta.min <- 1
theta.max <- 100
#pick parameter(s) from the initial distribution (uniform)
theta <- runif(niter,theta.min,theta.max)
write.table(x=theta,file="snm_thetap.txt",quote=F,row.name=F,col.name=F)

#Then, we will simulate 50000 iterations using ms, each iteration will calculate sequences from a coalescent tree with the θ value provided by us. We go to the folder that have the ms program and the file snm thetap.txt. ms needs at least the number of samples, the number of iterations and the value of θ for run. For example:

#In case you have the ms and sample_stats applications in the working directory:
#system execute a command like in a terminal console
system("./ms 20 2 -t 10")
#  run 2 iterations with a sample of 20 each and with a θ = 10. The output is a scalar giving the number of SNPs, a vector indicating the relative position of each SNP and a matrix with the presence or absence of the derived mutation. 
# Now we run the simulation using the prior distribution:

system("./ms 20 50000 -t tbs < ./snm_thetap.txt > ./SNM_coaldata.txt")
#tbs indicates that this parameter will be provided each time (here from a file)

#Finally we calculate several statistics using the program sample stats:

system("./sample_stats < ./SNM_coaldata.txt > ./SNM_statsdata.txt")
system("cut -f 2,4,8,11./SNM_statsdata.txt > ./SNM_statsdata_piSHprior.txt")
#we select only the columns for theta(Tajima), S, Theta(Fay&Wu) and the value of the prior

###################################################################################################
#3.2	Inference of the parameter θ of the SNM using ABC 
###################################################################################################
#We will use the R library abc. We go to R and do:

SNM <- read.table(file="SNM_statsdata_piSHprior.txt")
head(SNM)
colnames(SNM) <- c("Pi","S","H","Theta.prior")
head(SNM)
param.snm <- data.frame(theta=SNM[,4])

help(abc)
abc.snm <- abc(target=OBS,param=param.snm,sumstat=SNM[,-4],tol=0.01,method="loclinear")
summary(abc.snm)
plot(abc.snm,param=data.frame(theta=SNM[,4]))


###################################################################################################
3.3 Validating the parameter estimates
###################################################################################################
#Results can be validated from several ways. Here, the parameter inference can be validated by using ran- dom values from the simulation (all simulation values have known population parameter) and calculate the estimation error.

#Validation
snm.val.theta <- cv4abc(param.snm,sumstat=SNM[,-4],nval=100,tols=0.01,method="loclinear")
summary(snm.val.theta)
plot(snm.val.theta)

###################################################################################################
3.4	Parameter inference under the Demographic Expansion Model
###################################################################################################
#In that case, the exponential model has three parameters (theta, time to expansion process, strength of expansion). Remember in coalescent simulations all the events go from present to past. We do the same than before. (i) Define prior distributions for each parameter and calculate niter values; (ii) run coalescent simulations and obtain the summary statistics (iii) infer population parameter using the local lineal regression method from the R function abc; (iv) validate the inference values using a cross validation method.


####Expansion: 3 parameters: mutpop,time.exp,strength#####
#define min and max values for the priors
theta.exp.min <- 1
theta.exp.max <- 100
time.exp.min  <- 0
time.exp.max  <- 1
strg.exp.min  <- log10(1e-3)
strg.exp.max  <- log10(1)
	
#pick parameter(s) from the initial distribution
theta.exp <- runif(niter,theta.exp.min,theta.exp.max)
time.exp <- runif(niter,time.exp.min,time.exp.max)
strg.exp <- runif(niter,strg.exp.min,strg.exp.max)
strg.exp <- 10^(strg.exp) #This one is log10 scaled
write.table(x=cbind(theta.exp,time.exp,strg.exp),file="dem_p.txt",quote=F,row.name=F,col.name=F)

system("./ms 20 50000 -t tbs -eN tbs tbs < ./dem_p.txt > ./DEM_coaldata.txt")
system("./sample_stats < ./DEM_coaldata.txt > ./DEM_statsdata.txt")
system("cut -f 2,4,8,11,12,13 ./DEM_statsdata.txt > ./DEM_statsdata_piSHprior.txt")

DEM <- read.table(file="DEM_statsdata_piSHprior.txt")
head(DEM)
colnames(DEM) <- c("Pi","S","H","Theta.prior","Time_prior","Strength_prior")
head(DEM)
param.dem <- data.frame(theta=DEM[,4],time=DEM[,5],strength=log10(DEM[,6]))

abc.dem <- abc(target=OBS,param=param.dem,sumstat=DEM[,-c(4:6)],tol=0.01,method="loclinear",transf=c("none","none","none"))
summary(abc.dem)
plot(abc.dem,param=param.dem)




#Validation
dem.val.theta <- cv4abc(param.dem$theta,sumstat=DEM[,-c(4:6)],nval=100,tols=0.01,method="loclinear")
summary(dem.val.theta)
plot(dem.val.theta)
dem.val.time <-  cv4abc(param.dem$time,sumstat=DEM[,-c(4:6)],nval=100,tols=0.01,method="loclinear")
summary(dem.val.time)
plot(dem.val.time)
dem.val.strength <- cv4abc(param.dem$strength,sumstat=DEM[,-c(4:6)],nval=100,tols=0.01,method="loclinear")
summary(dem.val.strength)
plot(dem.val.strength)

###################################################################################################
3.5 Model Choice by ABC
###################################################################################################
#Model choice use the summary statistics of each model (SNM and DEM) to calculate the distance to the observed values. In that case parameters values are not considered but instead a name for each model is given (for example SNM, DEM) and deal as a categorical value. The best model will be that on that has more values close to the observed one. A local multiple logistic regression can also be used. The validation process is similar to the used with parameters: considering one or another model true from simulations, estimate the error in the model choice.


#Model choice:

index <- c(rep("SNM",niter),rep("DEM",niter))
sumstat.both <- rbind(SNM[,-4],DEM[,-c(4:6)])
mchoice <- postpr(target= OBS,index=index,sumstat=sumstat.both,tol=0.01,method="rejection")
summary(mchoice)

#Validation
mchoice.val <- cv4postpr(index,sumstat.both,nval=100,tol=0.01,method="rejection")
summary(mchoice.val)

